@@ -3,7 +3,7 @@
  *  formatted electronic mail messages using the SMTP protocol described
  *  in RFC 2821.
  *
- *  Copyright (C) 2001  Brian Stafford  <brian@stafford.uklinux.net>
+ *  Copyright (C) 2001,2002  Brian Stafford  <brian@stafford.uklinux.net>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,8 @@
 #include <time.h>
 #include <errno.h>
 
+#include <missing.h>
+
 #include "libesmtp-private.h"
 #include "headers.h"
 #include "htable.h"
@@ -563,10 +565,6 @@ destroy_header_table (smtp_message_t message)
 
   assert (message != NULL);
 
-  /* Take out the hash table */
-  h_destroy (message->hdr_action, NULL, NULL);
-  message->hdr_action = NULL;
-
   /* Take out the linked list */
   for (header = message->headers; header!= NULL; header = next)
     {
@@ -576,6 +574,14 @@ destroy_header_table (smtp_message_t message)
       free (header->header);
       free (header);
     }
+
+  /* Take out the hash table */
+  if (message->hdr_action != NULL)
+    {
+      h_destroy (message->hdr_action, NULL, NULL);
+      message->hdr_action = NULL;
+    }
+
   message->headers = message->end_headers = NULL;
 }
 
@@ -840,30 +846,33 @@ smtp_set_header_option (smtp_message_t message, const char *header,
       return 0;
     }
 
-  if (info->prohibit || (info->action->flags & (PROHIBIT | PRESERVE)))
+  /* Don't permit options to be set on headers that must pass intact or
+     which are prohibited. */
+  if (info->action->flags & (PROHIBIT | PRESERVE))
     {
       set_error (SMTP_ERR_INVAL);
       return 0;
     }
 
-  if (option == Hdr_OVERRIDE)
+  /* There is an odd quirk when setting options.  Setting an option for
+     the OPTIONAL headers known to libESMTP causes default values to be
+     generated automatically when not found in the message, so long as
+     there is no other reason to prevent them appearing in the message! */
+
+  /* Don't allow the user to set override on prohibited headers. */
+  if (option == Hdr_OVERRIDE && !info->prohibit)
     {
-      /* There is an odd quirk of the Hdr_OVERRIDE option.  Setting it
-         to false for the OPTIONAL headers known to libESMTP causes
-         default values to be generated automatically when not found in
-         the message! */
       va_start (alist, option);
       info->override = !!va_arg (alist, int);
       va_end (alist);
       return 1;
     }
-  if (option == Hdr_PROHIBIT)
+  /* Don't allow the user to prohibit required headers. */
+  if (option == Hdr_PROHIBIT && !(info->action->flags & REQUIRE))
     {
-      /* Can't set prohibit if a header of this name already set up
-         or the header is required. */
-      if (info->hdr != NULL || (info->action->flags & REQUIRE))
-        return 0;
-      info->prohibit = 1;
+      va_start (alist, option);
+      info->prohibit = !!va_arg (alist, int);
+      va_end (alist);
       return 1;
     }
 
